@@ -41,21 +41,24 @@ public:
 	~Dialog_Progress();
 	
 	sigc::signal< bool, Operation * > signal_apply_operation ;
+	sigc::signal< Glib::ustring > signal_get_libparted_version ;
 		
 private:
-	void update_operation_details( const Gtk::TreeRow & treerow, const OperationDetails & operation_details ) ;
+	void on_signal_update( const OperationDetail & operationdetail ) ;
+	void dispatcher_on_update_gui_elements() ;
 	void on_signal_show() ;
 	void on_expander_changed() ;
 	void on_cell_data_description( Gtk::CellRenderer * renderer, const Gtk::TreeModel::iterator & iter) ;
 	static void *static_pthread_apply_operation( void * p_dialog_progress ) ;
 	void on_cancel() ;
 	void on_save() ;
-	void echo_operation_details( const OperationDetails & operation_details, std::ofstream & out ) ;
+	void echo_operation_details( const OperationDetail & operation_detail, std::ofstream & out ) ;
 	
 	void on_response( int response_id ) ;
 	bool on_delete_event( GdkEventAny * event ) ;
 
 	Gtk::Label label_current ;
+	Gtk::Label label_current_sub ;
 	Gtk::ProgressBar progressbar_all, progressbar_current ;
 	Gtk::TreeView treeview_operations ;
 	Gtk::TreeRow treerow ;
@@ -65,32 +68,36 @@ private:
 	Glib::RefPtr<Gdk::Pixbuf> icon_execute ;
 	Glib::RefPtr<Gdk::Pixbuf> icon_succes ;
 	Glib::RefPtr<Gdk::Pixbuf> icon_error ;
+	Glib::RefPtr<Gdk::Pixbuf> icon_info ;
+	Glib::RefPtr<Gdk::Pixbuf> icon_n_a ;
 
 	Glib::RefPtr<Gtk::TreeStore> treestore_operations;
 	
 	struct treeview_operations_Columns : public Gtk::TreeModelColumnRecord             
 	{
 		Gtk::TreeModelColumn<Glib::ustring> operation_description;
-		Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> > operation_icon;
+		Gtk::TreeModelColumn<Glib::ustring> elapsed_time ;
 		Gtk::TreeModelColumn< Glib::RefPtr<Gdk::Pixbuf> > status_icon;
-		Gtk::TreeModelColumn<OperationDetails::Status> hidden_status ;
 				
 		treeview_operations_Columns() 
 		{ 
 			add( operation_description );
-			add( operation_icon );
+			add( elapsed_time );
 			add( status_icon ) ;
-			add( hidden_status ) ;
 		} 
 	};
 	treeview_operations_Columns treeview_operations_columns;
 	
 	std::vector<Operation *> operations ;
-	bool pulse, succes, cancel ;
+	OperationDetail operationdetail ;
+	bool running, succes, cancel, pulse ;
 	pthread_t pthread ;
 	double fraction ;
-	unsigned int t ;
+	unsigned int t, warnings ;
 	Glib::ustring str_temp ;
+
+	Glib::Dispatcher dispatcher_update_gui_elements ;
+	Glib::ustring label_current_sub_text ;
 };
 
 }//GParted
