@@ -1,4 +1,4 @@
-/* Copyright (C) 2004, 2005, 2006, 2007, 2008 Bart Hakvoort
+/* Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Bart Hakvoort
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -112,7 +112,8 @@ public:
 	
 	std::vector<Partition> logicals ;
 
-	bool strict ;
+	bool strict ;		//Indicator if start and end sectors must stay unchanged
+	bool strict_start ;	//Indicator if start sector must stay unchanged
 	
 private:
 	void sort_paths_and_remove_duplicates() ;
