@@ -17,6 +17,7 @@
  
 #include "../include/Win_GParted.h"
 #include "../include/GParted_Core.h"
+#include "../include/DMRaid.h"
 #include "../include/FS_Info.h"
 #include "../include/OperationCopy.h"
 #include "../include/OperationCreate.h"
@@ -126,11 +127,21 @@ void GParted_Core::find_supported_filesystems()
 	FILESYSTEMS .push_back( fs_xfs .get_filesystem_support() ) ;
 
 	FS *fs ;
+	//btrfs  FIXME:  Add full support when on-disk-format stabilized
+	fs = new( FS ) ;
+	fs ->filesystem = GParted::FS_BTRFS ;
+	FILESYSTEMS .push_back( * fs ) ;
+
 	//lvm2 physical volume -- not a file system
 	fs = new( FS ) ;
 	fs ->filesystem = GParted::FS_LVM2 ;
 	FILESYSTEMS .push_back( * fs ) ;
 
+	//luks encryption-- not a file system
+	fs = new( FS ) ;
+	fs ->filesystem = GParted::FS_LUKS ;
+	FILESYSTEMS .push_back( * fs ) ;
+
 	//unknown file system (default when no match is found)
 	fs = new( FS ) ;
 	fs ->filesystem = GParted::FS_UNKNOWN ;
@@ -148,6 +159,7 @@ void GParted_Core::set_devices( std::vector<Device> & devices )
 	devices .clear() ;
 	Device temp_device ;
 	FS_Info fs_info( true ) ;  //Refresh cache of file system information
+	DMRaid dmraid( true ) ;    //Refresh cache of dmraid device information
 	
 	init_maps() ;
 	
@@ -163,36 +175,52 @@ void GParted_Core::set_devices( std::vector<Device> & devices )
 		//
 		//try to find all available devices
 		std::ifstream proc_partitions( "/proc/partitions" ) ;
-    	if ( proc_partitions )
-    	{
-    		//parse device names from /proc/partitions
-    		std::string line ;
-    		std::string device ;
-    		while ( getline( proc_partitions, line ) )
-    		{
-    			//Whole disk devices are the ones we want.
-    			//Device names without a trailing digit refer to the whole disk.
-    			device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+([^0-9]+)$") ;
-    			//Device names that end with a #[^p]# are HP Smart Array Devices (disks)
-    			//E.g., device = /dev/cciss/c0d0, partition = /dev/cciss/c0d0p1
-    			if ( device == "" )
-    				device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+(.*[0-9]+[^p]{1}[0-9]+)$") ;
-    			if ( device != "" )
-    			{
-    				//try to have libparted detect the device and add it to the list
-    				device = "/dev/" + device;
-    				/*TO TRANSLATORS: looks like Scanning /dev/sda */ 
-    				set_thread_status_message( String::ucompose ( _("Scanning %1"), device ) ) ;
-    				ped_device_get( device .c_str() ) ;
-    			}
-    		}
-    		proc_partitions .close() ;
-    	}
-    	else
-    	{
-    		//file /proc/partitions doesn't exist so use libparted to probe devices
-    		ped_device_probe_all();
-    	}
+		if ( proc_partitions )
+		{
+			//parse device names from /proc/partitions
+			std::string line ;
+			std::string device ;
+			while ( getline( proc_partitions, line ) )
+			{
+				//Whole disk devices are the ones we want.
+				//Device names without a trailing digit refer to the whole disk.
+				device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+([^0-9]+)$") ;
+				//Recognize /dev/mmcblk* devices.
+				//E.g., device = /dev/mmcblk0, partition = /dev/mmcblk0p1
+				if ( device == "" )
+					device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+(mmcblk[0-9]+)$") ;
+				//Device names that end with a #[^p]# are HP Smart Array Devices (disks)
+				//E.g., device = /dev/cciss/c0d0, partition = /dev/cciss/c0d0p1
+				if ( device == "" )
+					device = Utils::regexp_label(line, "^[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+[0-9]+[\t ]+(.*[0-9]+[^p]{1}[0-9]+)$") ;
+				if ( device != "" )
+				{
+					//try to have libparted detect the device and add it to the list
+					device = "/dev/" + device;
+					/*TO TRANSLATORS: looks like Scanning /dev/sda */ 
+					set_thread_status_message( String::ucompose ( _("Scanning %1"), device ) ) ;
+					ped_device_get( device .c_str() ) ;
+				}
+			}
+			proc_partitions .close() ;
+
+			//Try to find all dmraid devices
+			if (dmraid .is_dmraid_supported() ) {
+				std::vector<Glib::ustring> dmraid_devices ;
+				dmraid .get_devices( dmraid_devices ) ;
+				for ( unsigned int k=0; k < dmraid_devices .size(); k++ ) {
+					set_thread_status_message( String::ucompose ( _("Scanning %1"), dmraid_devices[k] ) ) ;
+					dmraid .create_dev_map_entries( dmraid_devices[k] ) ;
+					settle_device( 1 ) ;
+					ped_device_get( dmraid_devices[k] .c_str() ) ;
+				}
+			}
+		}
+		else
+		{
+			//file /proc/partitions doesn't exist so use libparted to probe devices
+			ped_device_probe_all();
+		}
 
 		lp_device = ped_device_get_next( NULL );
 		while ( lp_device ) 
@@ -314,7 +342,7 @@ bool GParted_Core::snap_to_cylinder( const Device & device, Partition & partitio
 		} else {
 			diff = partition .sector_start % device .cylsize ;
 		}
-		if ( diff )
+		if ( diff && ! partition .strict_start  )
 		{
 			if ( diff < ( device .cylsize / 2 ) )
 				partition .sector_start -= diff ;
@@ -441,7 +469,15 @@ bool GParted_Core::set_disklabel( const Glib::ustring & device_path, const Glib:
 		
 		close_device_and_disk() ;
 	}
-	
+
+	//delete and recreate disk entries if dmraid
+	DMRaid dmraid ;
+	if ( return_value && dmraid .is_dmraid_device( device_path ) )
+	{
+		dmraid .purge_dev_map_entries( device_path ) ;
+		dmraid .create_dev_map_entries( device_path ) ;
+	}
+
 	return return_value ;	
 }
 	
@@ -550,6 +586,7 @@ void GParted_Core::init_maps()
 	fstab_info .clear() ;
 
 	read_mountpoints_from_file( "/proc/mounts", mount_info ) ;
+	read_mountpoints_from_file_swaps( "/proc/swaps", mount_info ) ;
 	read_mountpoints_from_file( "/etc/mtab", mount_info ) ;
 	read_mountpoints_from_file( "/etc/fstab", fstab_info ) ;
 	
@@ -623,6 +660,26 @@ void GParted_Core::read_mountpoints_from_file( const Glib::ustring & filename,
 	}
 }
 
+void GParted_Core::read_mountpoints_from_file_swaps(
+	const Glib::ustring & filename,
+	std::map< Glib::ustring, std::vector<Glib::ustring> > & map )
+{
+	std::string line ;
+	std::string node ;
+	
+	std::ifstream file( filename .c_str() ) ;
+	if ( file )
+	{
+		while ( getline( file, line ) )
+		{
+			node = Utils::regexp_label( line, "^(/[^ ]+)" ) ;
+			if ( node .size() > 0 )
+				map[ node ] .push_back( "" /* no mountpoint for swap */ ) ;
+		}
+		file .close() ;
+	}
+}
+
 std::vector<Glib::ustring> GParted_Core::get_alternate_paths( const Glib::ustring & path ) 
 {
 	std::vector<Glib::ustring> paths ;
@@ -639,7 +696,8 @@ void GParted_Core::set_device_partitions( Device & device )
 	int EXT_INDEX = -1 ;
 	char * lp_path ;//we have to free the result of ped_partition_get_path()..
 	FS_Info fs_info ;  //Use cache of file system information
-	
+	DMRaid dmraid ;    //Use cache of dmraid device information
+
 	//clear partitions
 	device .partitions .clear() ;
 
@@ -648,12 +706,31 @@ void GParted_Core::set_device_partitions( Device & device )
 	{
 		libparted_messages .clear() ;
 		partition_temp .Reset() ;
-	
+		bool partition_is_busy = false ;
+
 		switch ( lp_partition ->type )
 		{
 			case PED_PARTITION_NORMAL:
-			case PED_PARTITION_LOGICAL:  
+			case PED_PARTITION_LOGICAL:
 				lp_path = ped_partition_get_path( lp_partition ) ;
+
+				//Handle dmraid devices differently because the minor number might not
+				//  match the last number of the partition filename as shown by "ls -l /dev/mapper"
+				//  This mismatch causes incorrect identification of busy partitions in ped_partition_is_busy(). 
+				if ( dmraid .is_dmraid_device( device .get_path() ) )
+				{
+					//Try device_name + partition_number
+					iter_mp = mount_info .find( device .get_path() + Utils::num_to_str( lp_partition ->num ) ) ;
+					if ( iter_mp != mount_info .end() )
+						partition_is_busy = true ;
+					//Try device_name + p + partition_number
+					iter_mp = mount_info .find( device .get_path() + "p" + Utils::num_to_str( lp_partition ->num ) ) ;
+					if ( iter_mp != mount_info .end() )
+						partition_is_busy = true ;
+				}
+				else
+					partition_is_busy = ped_partition_is_busy( lp_partition ) ;
+
 				partition_temp .Set( device .get_path(),
 						     lp_path,
 						     lp_partition ->num,
@@ -662,7 +739,7 @@ void GParted_Core::set_device_partitions( Device & device )
 						     lp_partition ->geom .start,
 						     lp_partition ->geom .end,
 						     lp_partition ->type,
-						     ped_partition_is_busy( lp_partition ) ) ;
+						     partition_is_busy ) ;
 				free( lp_path ) ;
 						
 				partition_temp .add_paths( get_alternate_paths( partition_temp .get_path() ) ) ;
@@ -675,6 +752,27 @@ void GParted_Core::set_device_partitions( Device & device )
 			
 			case PED_PARTITION_EXTENDED:
 				lp_path = ped_partition_get_path( lp_partition ) ;
+
+				//Handle dmraid devices differently because the minor number might not
+				//  match the last number of the partition filename as shown by "ls -l /dev/mapper"
+				//  This mismatch causes incorrect identification of busy partitions in ped_partition_is_busy(). 
+				if ( dmraid .is_dmraid_device( device .get_path() ) )
+				{
+					for ( unsigned int k = 5; k < 255; k++ )
+					{
+						//Try device_name + [5 to 255]
+						iter_mp = mount_info .find( device .get_path() + Utils::num_to_str( k ) ) ;
+						if ( iter_mp != mount_info .end() )
+							partition_is_busy = true ;
+						//Try device_name + p + [5 to 255]
+						iter_mp = mount_info .find( device .get_path() + "p" + Utils::num_to_str( k ) ) ;
+						if ( iter_mp != mount_info .end() )
+							partition_is_busy = true ;
+					}
+				}
+				else
+					partition_is_busy = ped_partition_is_busy( lp_partition ) ;
+
 				partition_temp .Set( device .get_path(),
 					 	     lp_path, 
 						     lp_partition ->num,
@@ -683,7 +781,7 @@ void GParted_Core::set_device_partitions( Device & device )
 						     lp_partition ->geom .start,
 						     lp_partition ->geom .end,
 						     false,
-						     ped_partition_is_busy( lp_partition ) ) ;
+						     partition_is_busy ) ;
 				free( lp_path ) ;
 				
 				partition_temp .add_paths( get_alternate_paths( partition_temp .get_path() ) ) ;
@@ -736,11 +834,34 @@ void GParted_Core::set_device_partitions( Device & device )
 
 GParted::FILESYSTEM GParted_Core::get_filesystem() 
 {
+	char buf[512] ;
+	char magic[16] ;
+
+	//Check for LUKS encryption prior to libparted file system detection.
+	//  Otherwise encrypted file systems such as ext3 will be detected by
+	//  libparted as 'ext3'.
+
+	//LUKS encryption
+	ped_device_open( lp_device );
+	ped_geometry_read( & lp_partition ->geom, buf, 0, 1 ) ;
+	strncpy(magic, buf+0, 6) ;  magic[6] = '\0' ; //set and terminate string
+	ped_device_close( lp_device );
+
+	if ( Glib::ustring( magic ) == "LUKS\xBA\xBE" )
+	{
+		temp = _( "Linux Unified Key Setup encryption is not yet supported." ) ;
+		temp += "\n" ;
+		partition_temp .messages .push_back( temp ) ;
+		return GParted::FS_LUKS ;
+	}
+
 	//standard libparted file systems..
 	if ( lp_partition && lp_partition ->fs_type )
 	{
 		if ( Glib::ustring( lp_partition ->fs_type ->name ) == "extended" )
 			return GParted::FS_EXTENDED ;
+		else if ( Glib::ustring( lp_partition ->fs_type ->name ) == "btrfs" )
+			return GParted::FS_BTRFS ;
 		else if ( Glib::ustring( lp_partition ->fs_type ->name ) == "ext2" )
 			return GParted::FS_EXT2 ;
 		else if ( Glib::ustring( lp_partition ->fs_type ->name ) == "ext3" )
@@ -783,17 +904,15 @@ GParted::FILESYSTEM GParted_Core::get_filesystem()
 	
 	
 	//other file systems libparted couldn't detect (i've send patches for these file systems to the parted guys)
-	// - no patches sent to parted for lvm2
-	char buf[512] ;
-
-	ped_device_open( lp_device );
+	// - no patches sent to parted for lvm2, or luks
 
 	//reiser4
+	ped_device_open( lp_device );
 	ped_geometry_read( & lp_partition ->geom, buf, 128, 1 ) ;
 	ped_device_close( lp_device );
 	
 	if ( Glib::ustring( buf ) == "ReIsEr4" )
-		return GParted::FS_REISER4 ;		
+		return GParted::FS_REISER4 ;
 
 	//lvm2
 	//NOTE: lvm2 is not a file system but we do wish to recognize the Physical Volume
@@ -815,6 +934,29 @@ GParted::FILESYSTEM GParted_Core::get_filesystem()
 		return GParted::FS_LVM2 ;
 	}
 
+	//btrfs
+	const Sector BTRFS_SUPER_INFO_SIZE   = 4096 ;
+	const Sector BTRFS_SUPER_INFO_OFFSET = (64 * 1024) ;
+	const Glib::ustring BTRFS_SIGNATURE  = "_BHRfS_M" ;
+
+	char    buf_btrfs[BTRFS_SUPER_INFO_SIZE] ;
+
+	ped_device_open( lp_device ) ;
+	ped_geometry_read( & lp_partition ->geom, buf_btrfs, \
+	                   (BTRFS_SUPER_INFO_OFFSET / 512), \
+	                   (BTRFS_SUPER_INFO_SIZE / 512)
+	                 ) ;
+	strncpy(magic, buf_btrfs+64, BTRFS_SIGNATURE .size()) ;  magic[BTRFS_SIGNATURE .size()] = '\0' ; //set and terminate string
+	ped_device_close( lp_device ) ;
+
+	if ( magic == BTRFS_SIGNATURE )
+	{
+		temp = _( "BTRFS is not yet supported." ) ;
+		temp += "\n" ;
+		partition_temp .messages .push_back( temp ) ;
+		return GParted::FS_BTRFS ;
+	}
+
 	//no file system found....
 	temp = _( "Unable to detect file system! Possible reasons are:" ) ;
 	temp += "\n-"; 
@@ -899,22 +1041,51 @@ void GParted_Core::insert_unallocated( const Glib::ustring & device_path,
 	
 void GParted_Core::set_mountpoints( std::vector<Partition> & partitions ) 
 {
+	DMRaid dmraid ;	//Use cache of dmraid device information
 	for ( unsigned int t = 0 ; t < partitions .size() ; t++ )
 	{
 		if ( ( partitions[ t ] .type == GParted::TYPE_PRIMARY ||
-		       partitions[ t ] .type == GParted::TYPE_LOGICAL ) &&
+		       partitions[ t ] .type == GParted::TYPE_LOGICAL
+		     ) &&
 		     partitions[ t ] .filesystem != GParted::FS_LINUX_SWAP &&
-		     partitions[ t ] .filesystem != GParted::FS_LVM2 )
+		     partitions[ t ] .filesystem != GParted::FS_LVM2 &&
+		     partitions[ t ] .filesystem != GParted::FS_LUKS &&
+		     partitions[ t ] .filesystem != GParted::FS_BTRFS
+		   )
 		{
 			if ( partitions[ t ] .busy )
 			{
 				for ( unsigned int i = 0 ; i < partitions[ t ] .get_paths() .size() ; i++ )
 				{
-					iter_mp = mount_info .find( partitions[ t ] .get_paths()[ i ] ) ;
-					if ( iter_mp != mount_info .end() )
+					//Handle dmraid devices differently because there may be more
+					//  than one partition name.
+					//  E.g., there might be names with and/or without a 'p' between
+					//        the device name and partition number.
+					if ( dmraid .is_dmraid_device( partitions[ t ] .device_path ) )
 					{
-						partitions[ t ] .add_mountpoints( iter_mp ->second ) ;
-						break ;
+						//Try device_name + partition_number
+						iter_mp = mount_info .find( partitions[ t ] .device_path + Utils::num_to_str( partitions[ t ] .partition_number ) ) ;
+						if ( iter_mp != mount_info .end() )
+						{
+							partitions[ t ] .add_mountpoints( iter_mp ->second ) ;
+							break ;
+						}
+						//Try device_name + p + partition_number
+						iter_mp = mount_info .find( partitions[ t ] .device_path + "p" + Utils::num_to_str( partitions[ t ] .partition_number ) ) ;
+						if ( iter_mp != mount_info .end() )
+						{
+							partitions[ t ] .add_mountpoints( iter_mp ->second ) ;
+							break ;
+						}
+					}
+					else
+					{
+						iter_mp = mount_info .find( partitions[ t ] .get_paths()[ i ] ) ;
+						if ( iter_mp != mount_info .end() )
+						{
+							partitions[ t ] .add_mountpoints( iter_mp ->second ) ;
+							break ;
+						}
 					}
 				}
 
@@ -944,8 +1115,11 @@ void GParted_Core::set_used_sectors( std::vector<Partition> & partitions )
 	for ( unsigned int t = 0 ; t < partitions .size() ; t++ )
 	{
 		if ( partitions[ t ] .filesystem != GParted::FS_LINUX_SWAP &&
+		     partitions[ t ] .filesystem != GParted::FS_BTRFS &&
+		     partitions[ t ] .filesystem != GParted::FS_LUKS &&
 		     partitions[ t ] .filesystem != GParted::FS_LVM2 &&
-		     partitions[ t ] .filesystem != GParted::FS_UNKNOWN )
+		     partitions[ t ] .filesystem != GParted::FS_UNKNOWN
+		   )
 		{
 			if ( partitions[ t ] .type == GParted::TYPE_PRIMARY ||
 			     partitions[ t ] .type == GParted::TYPE_LOGICAL ) 
@@ -1133,16 +1307,16 @@ bool GParted_Core::create_partition( Partition & new_partition, OperationDetail
 		close_device_and_disk() ;
 	}
 
-	if ( new_partition .partition_number > 0 && erase_filesystem_signatures( new_partition ) )
-	{ 
-		operationdetail .get_last_child() .set_status( STATUS_SUCCES ) ;
-		return true ;
-	}
-	else
-	{		
-		operationdetail .get_last_child() .set_status( STATUS_ERROR ) ;	
-		return false ;
-	} 
+	bool succes = new_partition .partition_number > 0 && erase_filesystem_signatures( new_partition ) ;
+
+	//create dev map entries if dmraid
+	DMRaid dmraid ;
+	if ( succes && dmraid .is_dmraid_device( new_partition .device_path ) )
+		succes = dmraid .create_dev_map_entries( new_partition, operationdetail .get_last_child() ) ;
+
+	operationdetail .get_last_child() .set_status( succes ? STATUS_SUCCES : STATUS_ERROR ) ;
+
+	return succes ;
 }
 	
 bool GParted_Core::create_filesystem( const Partition & partition, OperationDetail & operationdetail ) 
@@ -1197,6 +1371,23 @@ bool GParted_Core::Delete( const Partition & partition, OperationDetail & operat
 		close_device_and_disk() ;
 	}
 
+	//delete partition dev mapper entry, and delete and recreate all other affected dev mapper entries if dmraid
+	DMRaid dmraid ;
+	if ( succes && dmraid .is_dmraid_device( partition .device_path ) )
+	{
+		//Open disk handle before and close after to prevent application crash.
+		if ( open_device_and_disk( partition .device_path ) )
+		{
+			if ( ! dmraid .delete_affected_dev_map_entries( partition, operationdetail .get_last_child() ) )
+				succes = false ;	//comand failed
+
+			if ( ! dmraid .create_dev_map_entries( partition, operationdetail .get_last_child() ) )
+				succes = false ;	//command failed
+
+			close_device_and_disk() ;
+		}
+	}
+
 	operationdetail .get_last_child() .set_status( succes ? STATUS_SUCCES : STATUS_ERROR ) ;
 	return succes ;
 }
@@ -1242,7 +1433,10 @@ bool GParted_Core::resize_move( const Device & device,
 			  	Partition & partition_new,
 			  	OperationDetail & operationdetail ) 
 {
-	if ( partition_new .strict || calculate_exact_geom( partition_old, partition_new, operationdetail ) )
+	if (   partition_new .strict
+		|| partition_new .strict_start
+		|| calculate_exact_geom( partition_old, partition_new, operationdetail )
+	   )
 	{
 		if ( partition_old .type == TYPE_EXTENDED )
 			return resize_move_partition( partition_old, partition_new, operationdetail ) ;
@@ -1299,8 +1493,8 @@ bool GParted_Core::move( const Device & device,
 		//(maybe i should do some reading on how non-msdos disklabels deal with metadata....)
 		if ( partition_new .sector_start < partition_old .sector_start )
 		{
-	       		if ( resize_move_partition( partition_old, partition_new, operationdetail ) )
-	       	   	{
+			if ( resize_move_partition( partition_old, partition_new, operationdetail ) )
+			{
 				if ( ! move_filesystem( partition_old, partition_new, operationdetail ) )
 				{
 					operationdetail .add_child( OperationDetail( _("rollback last change to the partition table") ) ) ;
@@ -1315,13 +1509,13 @@ bool GParted_Core::move( const Device & device,
 			}
 		}
 		else
-	       		succes = move_filesystem( partition_old, partition_new, operationdetail ) &&
-	       			 resize_move_partition( partition_old, partition_new, operationdetail ) ;
+			succes = move_filesystem( partition_old, partition_new, operationdetail ) &&
+				resize_move_partition( partition_old, partition_new, operationdetail ) ;
 
 		succes = succes &&
-			 update_bootsector( partition_new, operationdetail ) &&
-	       		 check_repair_filesystem( partition_new, operationdetail ) &&
-	       		 maximize_filesystem( partition_new, operationdetail ) ;
+			update_bootsector( partition_new, operationdetail ) &&
+			check_repair_filesystem( partition_new, operationdetail ) &&
+			maximize_filesystem( partition_new, operationdetail ) ;
 	}
 
 	return succes ;
@@ -1581,10 +1775,14 @@ bool GParted_Core::resize_move_partition( const Partition & partition_old,
 		
 		if ( lp_partition )
 		{
-			PedGeometry *geom = ped_geometry_new( lp_device,
-							      partition_new .sector_start,
-							      partition_new .get_length() ) ;
-			constraint = ped_constraint_exact( geom ) ;
+			if ( partition_new .strict || partition_new .strict_start ) {
+				PedGeometry *geom = ped_geometry_new( lp_device,
+									  partition_new .sector_start,
+									  partition_new .get_length() ) ;
+				constraint = ped_constraint_exact( geom ) ;
+			}
+			else
+				constraint = ped_constraint_any( lp_device ) ;
 
 			if ( constraint )
 			{
@@ -1618,6 +1816,18 @@ bool GParted_Core::resize_move_partition( const Partition & partition_old,
 					  	  Utils::format_size( new_end - new_start + 1 ) ),
 			STATUS_NONE, 
 			FONT_ITALIC ) ) ;
+
+		//update dev mapper entry if partition is dmraid.
+		DMRaid dmraid ;
+		if ( return_value && dmraid .is_dmraid_device( partition_new .device_path ) )
+		{
+			//Open disk handle before and close after to prevent application crash.
+			if ( open_device_and_disk( partition_new .device_path ) )
+			{
+				return_value = dmraid .update_dev_map_entry( partition_new, operationdetail .get_last_child() ) ;
+				close_device_and_disk() ;
+			}
+		}
 	}
 	
 	operationdetail .get_last_child() .set_status( return_value ? STATUS_SUCCES : STATUS_ERROR ) ;
@@ -2280,6 +2490,18 @@ bool GParted_Core::calculate_exact_geom( const Partition & partition_old,
 					  	  Utils::format_size( partition_new .get_length() ) ),
 			STATUS_NONE,
 			FONT_ITALIC ) ) ;
+
+		//Update dev mapper entry if partition is dmraid.
+		DMRaid dmraid ;
+		if ( succes && dmraid .is_dmraid_device( partition_new .device_path ) )
+		{
+			//Open disk handle before and close after to prevent application crash.
+			if ( open_device_and_disk( partition_new .device_path ) )
+			{
+				succes = dmraid .update_dev_map_entry( partition_new, operationdetail .get_last_child() ) ;
+				close_device_and_disk() ;
+			}
+		}
 	}
 
 	operationdetail .get_last_child() .set_status( succes ? STATUS_SUCCES : STATUS_ERROR ) ;
@@ -2438,16 +2660,28 @@ bool GParted_Core::commit()
 
 bool GParted_Core::commit_to_os( std::time_t timeout ) 
 {
-	bool succes = ped_disk_commit_to_os( lp_disk ) ;
-
-	if ( Glib::find_program_in_path( "udevsettle" ) .empty() )
-		sleep( timeout ) ;
+	DMRaid dmraid ;
+	bool succes ;
+	if ( dmraid .is_dmraid_device( lp_disk ->dev ->path ) )
+		succes = true ;
 	else
-		Utils::execute_command( "udevsettle --timeout=" + Utils::num_to_str( timeout ) ) ;
+		succes = ped_disk_commit_to_os( lp_disk ) ;
+
+	settle_device( timeout ) ;
 
 	return succes ;
 }
-	
+
+void GParted_Core::settle_device( std::time_t timeout )
+{
+	if ( ! Glib::find_program_in_path( "udevsettle" ) .empty() )
+		Utils::execute_command( "udevsettle --timeout=" + Utils::num_to_str( timeout ) ) ;
+	else if ( ! Glib::find_program_in_path( "udevadm" ) .empty() )
+		Utils::execute_command( "udevadm settle --timeout=" + Utils::num_to_str( timeout ) ) ;
+	else
+		sleep( timeout ) ;
+}
+
 PedExceptionOption GParted_Core::ped_exception_handler( PedException * e ) 
 {
         std::cout << e ->message << std::endl ;
