@@ -1,4 +1,4 @@
-/* Copyright (C) 2004, 2005, 2006, 2007, 2008 Bart Hakvoort
+/* Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009 Bart Hakvoort
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 #include <regex.h>
 #include <locale.h>
 
+
 namespace GParted
 {
 
@@ -71,6 +72,7 @@ Glib::ustring Utils::get_color( FILESYSTEM filesystem )
 		case FS_UNKNOWN		: return "#000000" ;	//black
 		case FS_UNFORMATTED	: return "#000000" ;	//black
 		case FS_EXTENDED	: return "#7DFCFE" ;	// ~ light blue
+		case FS_BTRFS		: return "#FF9955" ;	//orange
 		case FS_EXT2		: return "#9DB8D2" ;	//blue hilight
 		case FS_EXT3		: return "#7590AE" ;	//blue medium
 		case FS_EXT4		: return "#4B6983" ;	//blue dark
@@ -88,6 +90,7 @@ Glib::ustring Utils::get_color( FILESYSTEM filesystem )
 		case FS_USED		: return "#F8F8BA" ;	// ~ light tan yellow
 		case FS_UNUSED		: return "#FFFFFF" ;	//white
 		case FS_LVM2		: return "#CC9966" ;	// ~ medium brown
+		case FS_LUKS		: return "#625B81" ;	//purple dark
 
 		default				: return "#000000" ;
 	}
@@ -117,6 +120,7 @@ Glib::ustring Utils::get_filesystem_string( FILESYSTEM filesystem )
 		case FS_UNKNOWN		: return _("unknown") ;
 		case FS_UNFORMATTED	: return _("unformatted") ;
 		case FS_EXTENDED	: return "extended" ;
+		case FS_BTRFS		: return "btrfs" ;
 		case FS_EXT2		: return "ext2" ;
 		case FS_EXT3		: return "ext3" ;
 		case FS_EXT4		: return "ext4" ;
@@ -134,6 +138,7 @@ Glib::ustring Utils::get_filesystem_string( FILESYSTEM filesystem )
 		case FS_USED		: return _("used") ;
 		case FS_UNUSED		: return _("unused") ;
 		case FS_LVM2		: return "lvm2" ;
+		case FS_LUKS		: return "crypt-luks" ;
 
 		default			: return "" ;
 	}
@@ -414,5 +419,27 @@ Glib::ustring Utils::get_lang()
 	return lang ;
 }
 
+//The tokenize method copied and adapted from:
+//  http://www.linuxselfhelp.com/HOWTO/C++Programming-HOWTO-7.html
+void Utils::tokenize( const Glib::ustring& str,
+                      std::vector<Glib::ustring>& tokens,
+                      const Glib::ustring& delimiters = " " )
+{
+	// Skip delimiters at beginning.
+	Glib::ustring::size_type lastPos = str.find_first_not_of(delimiters, 0);
+	// Find first "non-delimiter".
+	Glib::ustring::size_type pos     = str.find_first_of(delimiters, lastPos);
+
+	while (Glib::ustring::npos != pos || Glib::ustring::npos != lastPos)
+	{
+		// Found a token, add it to the vector.
+		tokens.push_back(str.substr(lastPos, pos - lastPos));
+		// Skip delimiters.  Note the "not_of"
+		lastPos = str.find_first_not_of(delimiters, pos);
+		// Find next "non-delimiter"
+		pos = str.find_first_of(delimiters, lastPos);
+	}
+}
+
 
 } //GParted..
