@@ -385,9 +385,12 @@ Gtk::Menu * Win_GParted::create_format_menu()
 
 	for ( unsigned int t =0; t < gparted_core .get_filesystems() .size() ; t++ )
 	{
-		//Skip lvm2 and unknown because these are not file systems
-		if( gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_LVM2 ||
-			gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_UNKNOWN )
+		//Skip btrfs, luks, lvm2, and unknown because these are not file systems
+		if ( gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_BTRFS ||
+		     gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_LUKS ||
+		     gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_LVM2 ||
+		     gparted_core .get_filesystems()[ t ] .filesystem == GParted::FS_UNKNOWN
+		   )
 			continue ;
 
 		hbox = manage( new Gtk::HBox() );
@@ -1603,8 +1606,8 @@ void Win_GParted::activate_format( GParted::FILESYSTEM new_fs )
 	//check for some limits...
 	fs = gparted_core .get_fs( new_fs ) ;
 	
-	if ( selected_partition .get_length() < fs .MIN ||
-	     fs .MAX && selected_partition .get_length() > fs .MAX ) 
+	if ( ( selected_partition .get_length() < fs .MIN ) ||
+	     ( fs .MAX && selected_partition .get_length() > fs .MAX ) )
 	{
 		Gtk::MessageDialog dialog( *this,
 					   String::ucompose( _("Cannot format this file system to %1."),
